!----------------------------------------------------------------------
!
!  File:   ice_vars.f90
!  Author: Ian Ross, University of Bristol
!  Date:   20 Feb 2005
!
!----------------------------------------------------------------------
!
!  Routines for fetching and calculating daily sea ice variable output
!  from FOAM/LPJ.
!
!----------------------------------------------------------------------

MODULE ice_vars

  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils

  USE foam_defs                 ! FOAM specific definitions.
  USE file_utils
  USE math_utils


  IMPLICIT NONE

  PRIVATE
  PUBLIC :: get_sea_ice_area
  PUBLIC :: get_sea_ice_extent
  PUBLIC :: get_sea_ice_volume
  PUBLIC :: get_sea_ice_basal_heat_flux
  PUBLIC :: get_sea_ice_surf_temp

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Get interval averaged sea ice area.
  !

  SUBROUTINE get_sea_ice_area(sday, eday, data)

    IMPLICIT NONE

    INTEGER,                      INTENT(IN)  :: sday, eday

    REAL, DIMENSION(:),           INTENT(OUT) :: data


    REAL, DIMENSION(:), ALLOCATABLE, SAVE    :: lat_vals, lon_vals
    REAL, DIMENSION(:, :), ALLOCATABLE, SAVE :: lat_bnds, lon_bnds
    LOGICAL, SAVE                            :: first = .TRUE.

    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: sea_ice_conc
    REAL, DIMENSION(2)                    :: tmp_data
    INTEGER :: day, hemi
    INTEGER :: latidx, lonidx
    INTEGER :: nc_id
    REAL :: dlat, dlon, lat, single_area


    ! Get longitude and latitude values and bounds.

    IF (first) THEN
       CALL lat_lon_grid(DOMAIN_ICE, AXIS_LON, lon_vals, lon_bnds)
       CALL lat_lon_grid(DOMAIN_ICE, AXIS_LAT, lat_vals, lat_bnds)
       first = .FALSE.
    END IF


    ! Accumulate daily data.

    data = 0
    DO day = sday, eday
       ! Get the sea ice concentration for today.

       nc_id = open_history_file('sic', day)
       CALL read_2d_netcdf_var(nc_id, 'sic', sea_ice_conc)
       CALL close_file(nc_id)


       ! Calculate the area for the two hemispheres separately.

       tmp_data = 0
       DO latidx = 1, FOAM_OLAT
          DO lonidx = 1, FOAM_OLON
             IF (sea_ice_conc(lonidx, latidx) == 0.0 .OR. &
                 sea_ice_conc(lonidx, latidx) == FOAM_MISSING_VAL) CYCLE

             IF (latidx <= FOAM_OLAT / 2) THEN
                hemi = SOUTHERN_HEMI
             ELSE
                hemi = NORTHERN_HEMI
             END IF

             dlat = D_TO_R * (lat_bnds(2, latidx) - lat_bnds(1, latidx))
             dlon = D_TO_R * (lon_bnds(2, lonidx) - lon_bnds(1, lonidx))
             lat = D_TO_R * lat_vals(latidx)
             single_area = R_EARTH**2 * COS(lat) * dlat * dlon

             tmp_data(hemi) = tmp_data(hemi) + &
                  sea_ice_conc(lonidx, latidx) * single_area
          END DO
       END DO


       ! Accumulate daily data.

       data = data + tmp_data
    END DO
    data = data / (eday - sday + 1)

  END SUBROUTINE get_sea_ice_area


  !--------------------------------------------------------------------
  !
  !  Get interval averaged sea ice extent.  This is defined as the total
  !  area of grid squares with more than 15% sea ice.
  !

  SUBROUTINE get_sea_ice_extent(sday, eday, data)

    IMPLICIT NONE

    INTEGER,                      INTENT(IN)  :: sday, eday

    REAL, DIMENSION(:),           INTENT(OUT) :: data


    REAL, DIMENSION(:), ALLOCATABLE, SAVE    :: lat_vals, lon_vals
    REAL, DIMENSION(:, :), ALLOCATABLE, SAVE :: lat_bnds, lon_bnds
    LOGICAL, SAVE                            :: first = .TRUE.

    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: sea_ice_conc
    REAL, DIMENSION(2)                    :: tmp_data
    INTEGER :: day, hemi
    INTEGER :: latidx, lonidx
    INTEGER :: nc_id
    REAL :: dlat, dlon, lat, single_area


    ! Get longitude and latitude values and bounds.

    IF (first) THEN
       CALL lat_lon_grid(DOMAIN_ICE, AXIS_LON, lon_vals, lon_bnds)
       CALL lat_lon_grid(DOMAIN_ICE, AXIS_LAT, lat_vals, lat_bnds)
       first = .FALSE.
    END IF


    ! Accumulate daily data.

    data = 0
    DO day = sday, eday
       ! Get the sea ice concentration for today.

       nc_id = open_history_file('sic', day)
       CALL read_2d_netcdf_var(nc_id, 'sic', sea_ice_conc)
       CALL close_file(nc_id)


       ! Calculate the extent for the two hemispheres separately.

       tmp_data = 0
       DO latidx = 1, FOAM_OLAT
          DO lonidx = 1, FOAM_OLON
             IF (sea_ice_conc(lonidx, latidx) < 0.15 .OR. &
                 sea_ice_conc(lonidx, latidx) == FOAM_MISSING_VAL) CYCLE

             IF (latidx <= FOAM_OLAT / 2) THEN
                hemi = SOUTHERN_HEMI
             ELSE
                hemi = NORTHERN_HEMI
             END IF

             dlat = D_TO_R * (lat_bnds(2, latidx) - lat_bnds(1, latidx))
             dlon = D_TO_R * (lon_bnds(2, lonidx) - lon_bnds(1, lonidx))
             lat = D_TO_R * lat_vals(latidx)
             single_area = R_EARTH**2 * COS(lat) * dlat * dlon

             tmp_data(hemi) = tmp_data(hemi) + single_area
          END DO
       END DO


       ! Accumulate daily data.

       data = data + tmp_data
    END DO
    data = data / (eday - sday + 1)

  END SUBROUTINE get_sea_ice_extent


  !--------------------------------------------------------------------
  !
  !  Get interval averaged sea ice volume.
  !

  SUBROUTINE get_sea_ice_volume(sday, eday, data)

    IMPLICIT NONE

    INTEGER,                      INTENT(IN)  :: sday, eday

    REAL, DIMENSION(:),           INTENT(OUT) :: data


    REAL, DIMENSION(:), ALLOCATABLE, SAVE    :: lat_vals, lon_vals
    REAL, DIMENSION(:, :), ALLOCATABLE, SAVE :: lat_bnds, lon_bnds
    LOGICAL, SAVE                            :: first = .TRUE.

    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: sea_ice_conc
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: sea_ice_thick
    REAL, DIMENSION(2)                    :: tmp_data
    INTEGER :: day, hemi
    INTEGER :: latidx, lonidx
    INTEGER :: nc_id
    REAL :: dlat, dlon, lat, single_area


    ! Get longitude and latitude values and bounds.

    IF (first) THEN
       CALL lat_lon_grid(DOMAIN_ICE, AXIS_LON, lon_vals, lon_bnds)
       CALL lat_lon_grid(DOMAIN_ICE, AXIS_LAT, lat_vals, lat_bnds)
       first = .FALSE.
    END IF


    ! Accumulate daily data.

    data = 0
    DO day = sday, eday
       ! Get the sea ice concentration and thickness for today.

       nc_id = open_history_file('sic', day)
       CALL read_2d_netcdf_var(nc_id, 'sic', sea_ice_conc)
       CALL read_2d_netcdf_var(nc_id, 'sit', sea_ice_thick)
       CALL close_file(nc_id)


       ! Calculate the volume for the two hemispheres separately.

       tmp_data = 0
       DO latidx = 1, FOAM_OLAT
          DO lonidx = 1, FOAM_OLON
             IF (sea_ice_conc(lonidx, latidx) == 0.0 .OR. &
                 sea_ice_conc(lonidx, latidx) == FOAM_MISSING_VAL) CYCLE

             IF (latidx <= FOAM_OLAT / 2) THEN
                hemi = SOUTHERN_HEMI
             ELSE
                hemi = NORTHERN_HEMI
             END IF

             dlat = D_TO_R * (lat_bnds(2, latidx) - lat_bnds(1, latidx))
             dlon = D_TO_R * (lon_bnds(2, lonidx) - lon_bnds(1, lonidx))
             lat = D_TO_R * lat_vals(latidx)
             single_area = R_EARTH**2 * COS(lat) * dlat * dlon

             tmp_data(hemi) = tmp_data(hemi) + single_area * &
                  sea_ice_conc(lonidx, latidx) * &
                  sea_ice_thick(lonidx, latidx) * 1.0E-3
          END DO
       END DO


       ! Accumulate daily data.

       data = data + tmp_data
    END DO
    data = data / (eday - sday + 1)

  END SUBROUTINE get_sea_ice_volume

  !--------------------------------------------------------------------
  !
  !  Get basal heat flux beneath sea ice.
  !

  SUBROUTINE get_sea_ice_basal_heat_flux(sday, eday, data)

    IMPLICIT NONE

    INTEGER,                      INTENT(IN)  :: sday, eday

    REAL, DIMENSION(:,:),         INTENT(OUT) :: data


    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: sea_ice_conc
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: avg_sic
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: ocean_heat_flux
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: avg_hfo
    
    INTEGER :: day, hfo_sday, hfo_eday
    INTEGER :: nc_id


    ! First, average the daily sea ice concentration data.

    avg_sic = 0
    DO day = sday, eday
       nc_id = open_history_file('sic', day)
       CALL read_2d_netcdf_var(nc_id, 'sic', sea_ice_conc)
       CALL close_file(nc_id)
       IF (day == sday) THEN
          avg_sic = sea_ice_conc
       ELSE
          WHERE (sea_ice_conc /= FOAM_MISSING_VAL) &
               avg_sic = avg_sic + sea_ice_conc
       END IF
    END DO
    WHERE (avg_sic /= FOAM_MISSING_VAL) &
         avg_sic = avg_sic / (eday - sday + 1)

    ! Now get the average oceanic heat flux over the same period.

    avg_hfo = 0
    hfo_sday = sday / 30 * 30 + 30
    hfo_eday = eday / 30 * 30
    DO day = hfo_sday, hfo_eday, 30
       nc_id = open_history_file('hfo', day)
       CALL read_2d_netcdf_var(nc_id, 'hfo', ocean_heat_flux)
       CALL close_file(nc_id)
       IF (day == hfo_sday) THEN
          avg_hfo = ocean_heat_flux
       ELSE
          WHERE (ocean_heat_flux /= FOAM_MISSING_VAL) &
               avg_hfo = avg_hfo + ocean_heat_flux
       END IF
    END DO
    WHERE (avg_hfo /= FOAM_MISSING_VAL) &
         avg_hfo = avg_hfo / (hfo_eday - hfo_sday + 30) / 30

    ! Set up the output data.

    data = FOAM_MISSING_VAL
    WHERE (avg_sic > 0) data = -avg_hfo

  END SUBROUTINE get_sea_ice_basal_heat_flux


  !--------------------------------------------------------------------
  !
  !  Get surface temperature of sea ice.
  !

  SUBROUTINE get_sea_ice_surf_temp(sday, eday, data)

    IMPLICIT NONE

    INTEGER,                      INTENT(IN)  :: sday, eday

    REAL, DIMENSION(:,:),         INTENT(OUT) :: data


    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: sea_ice_conc
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: avg_sic
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT) :: surf_temp_i
    REAL, DIMENSION(FOAM_ALON, FOAM_ALAT) :: surf_temp_a
    
    INTEGER :: day
    INTEGER :: nc_id

    REAL, DIMENSION(:), ALLOCATABLE :: alons, alats, ilons, ilats
    REAL, DIMENSION(:,:), ALLOCATABLE :: adum1, adum2, idum1, idum2


    ! Get latitude and longitude grids for regridding.

    CALL lat_lon_grid(DOMAIN_ATMOS, AXIS_LON, alons, adum1)
    CALL lat_lon_grid(DOMAIN_ATMOS, AXIS_LAT, alats, adum2)
    CALL lat_lon_grid(DOMAIN_ICE,   AXIS_LON, ilons, idum1)
    CALL lat_lon_grid(DOMAIN_ICE,   AXIS_LAT, ilats, idum2)


    ! First, average the daily sea ice concentration data.

    avg_sic = 0
    data = 0
    DO day = sday, eday
       nc_id = open_history_file('sic', day)
       CALL read_2d_netcdf_var(nc_id, 'sic', sea_ice_conc)
       CALL close_file(nc_id)
       nc_id = open_history_file('ts', day)
       CALL read_2d_netcdf_var(nc_id, 'ts', surf_temp_a)
       CALL close_file(nc_id)
       CALL regrid(surf_temp_a, alons, alats, surf_temp_i, ilons, ilats)
       IF (day == sday) THEN
          avg_sic = sea_ice_conc
          data = surf_temp_i
       ELSE
          WHERE (sea_ice_conc /= FOAM_MISSING_VAL)
             avg_sic = avg_sic + sea_ice_conc
          ELSEWHERE
             avg_sic = FOAM_MISSING_VAL
          END WHERE
          data = data + surf_temp_i
       END IF
    END DO
    WHERE (avg_sic /= FOAM_MISSING_VAL) &
         avg_sic = avg_sic / (eday - sday + 1)
    data = data / (eday - sday + 1)
    WHERE (avg_sic == FOAM_MISSING_VAL .OR. avg_sic == 0.0) &
         data = FOAM_MISSING_VAL

  END SUBROUTINE get_sea_ice_surf_temp

END MODULE ice_vars

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
