!----------------------------------------------------------------------
!
!  File:   math_utils.f90
!  Author: Ian Ross, University of Bristol
!  Date:   17 Feb 2005
!
!----------------------------------------------------------------------
!
!  Maths utility routines for FOAM model-specific code.
!
!----------------------------------------------------------------------

MODULE math_utils

  USE pp_defs
  USE pp_utils
  USE foam_defs

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: lat_lon_grid
  PUBLIC :: press_interp
  PUBLIC :: regrid

CONTAINS


  !--------------------------------------------------------------------
  !
  !  Calculate latitude and longitude grids for FOAM.
  !

  SUBROUTINE lat_lon_grid(domain, axis, values, bounds)

    IMPLICIT NONE

    CHARACTER, INTENT(IN) :: domain
    CHARACTER(LEN=AXIS_NAME_LEN), INTENT(IN) :: axis

    REAL, DIMENSION(:), ALLOCATABLE, INTENT(OUT) :: values
    REAL, DIMENSION(:, :), ALLOCATABLE, INTENT(OUT) :: bounds

    INTEGER :: i


    SELECT CASE (axis)

       ! Longitude axes: both the atmosphere and the ocean use
       ! regular longitude axes; the ocean grid is also used by the
       ! surface and ice models.
       
    CASE (AXIS_LON)
       SELECT CASE (domain)
       CASE (DOMAIN_ATMOS)
          ALLOCATE(values(FOAM_ALON))
          values = 7.5 * (/ (i, i = 0, FOAM_ALON - 1) /)
          ALLOCATE(bounds(2, FOAM_ALON))
          bounds(1, :) = values - 7.5 / 2.0
          bounds(2, :) = values + 7.5 / 2.0

       CASE (DOMAIN_OCEAN, DOMAIN_SURFACE, DOMAIN_ICE)
          ALLOCATE(values(FOAM_OLON))
          values = 360.0 / 128.0 * ((/ (i, i = 0, FOAM_OLON - 1) /) + 0.5)
          ALLOCATE(bounds(2, FOAM_OLON))
          bounds(1, :) = values - 360.0 / 128.0 / 2.0
          bounds(2, :) = values + 360.0 / 128.0 / 2.0
       END SELECT

       ! Latitude axes: the atmosphere uses a Gaussian latitude
       ! grid; the ocean uses a regular grid, and the ocean grid is
       ! also used for the surface and ice models.

    CASE (AXIS_LAT)
       SELECT CASE (domain)
       CASE (DOMAIN_ATMOS)
          ALLOCATE(values(FOAM_ALAT))
          CALL gaussian_grid(values)
          ALLOCATE(bounds(2, FOAM_ALAT))
          bounds(1, 1) = -90.0
          bounds(1, 2:) = (values(2:) + values(1:FOAM_ALAT-1)) / 2.0
          bounds(2, FOAM_ALAT) = 90.0
          bounds(2, 1:FOAM_ALAT-1) = bounds(1, 2:)
          
       CASE (DOMAIN_OCEAN, DOMAIN_SURFACE, DOMAIN_ICE)
          ALLOCATE(values(FOAM_OLAT))
          values = 180.0 / 128.0 * ((/ (i, i = 0, FOAM_OLAT - 1) /) - 64 + 0.5)
          ALLOCATE(bounds(2, FOAM_OLAT))
          bounds(1, 1) = -90.0
          bounds(1, 2:) = (values(2:) + values(1:FOAM_OLAT-1)) / 2.0
          bounds(2, FOAM_OLAT) = 90.0
          bounds(2, 1:FOAM_OLAT-1) = bounds(1, 2:)
       END SELECT
    END SELECT

  END SUBROUTINE lat_lon_grid


  !--------------------------------------------------------------------
  !
  !  Interpolation between pressure levels using log interpolation.
  !

  SUBROUTINE press_interp(out, in, pin, pout)

    IMPLICIT NONE

    REAL, DIMENSION(:),         INTENT(OUT) :: out   ! Output data.

    REAL, DIMENSION(:),         INTENT(IN)  :: in    ! Input data.
    REAL, DIMENSION(SIZE(in)),  INTENT(IN)  :: pin   ! Input pressure levels.
    REAL, DIMENSION(SIZE(out)), INTENT(IN)  :: pout  ! Output pressure levels.


    INTEGER :: outlev, icheck
    INTEGER :: below
    LOGICAL :: exact
    REAL :: po, p1, p2
    REAL :: in1, in2


    DO outlev = 1, SIZE(out)
       po = pout(outlev)

       ! Find bracketing input pressures.

       exact = .FALSE.
       below = 0
       DO icheck = 1, SIZE(in) - 1
          IF (pin(icheck) == po) THEN
             below = icheck
             exact = .TRUE.
             EXIT
          END IF
          IF (pin(icheck) < po .AND. pin(icheck + 1) > po) THEN
             below = icheck
             EXIT
          END IF
       END DO
       IF (pin(SIZE(in)) == po) THEN
          below = SIZE(in)
          exact = .TRUE.
       END IF


       ! Do interpolation: values of output pressure outside of the input
       ! pressure range produce missing values (they may, for instance,
       ! correspond to instances where the pressure level reaches the
       ! Earth's surface).

       IF (below == 0) THEN
          out(outlev) = FOAM_MISSING_VAL
       ELSE IF (exact) THEN
          out(outlev) = in(below)
       ELSE
          p1 = pin(below)     ; in1 = in(below)
          p2 = pin(below + 1) ; in2 = in(below + 1)
          out(outlev) = in1 + LOG(p1 / po) * (in2 - in1) / LOG(p1 / p2)
       END IF
    END DO

  END SUBROUTINE press_interp


  !--------------------------------------------------------------------
  !
  !  Calculate abscissae a(k) for Gaussian integration.  The zeros of the
  !  Bessel function j0 are used to get starting approximations for the
  !  abscissas.
  !

  SUBROUTINE gaussian_grid(absc, wght)

    IMPLICIT NONE

    REAL, DIMENSION(:), INTENT(OUT)           :: absc
    REAL, DIMENSION(:), INTENT(OUT), OPTIONAL :: wght

    REAL, PARAMETER :: pi = 3.14159265358979
    REAL, PARAMETER :: eps = 1.0E-13

    REAL :: c, xz, pkm1, pkm2, pk, sp, avsp
    INTEGER :: fk, kk, is, iter, n, fn

    ! Newton iteration is used to find the abscissas.

    c = (1.0 - (2.0 / pi)**2) * 0.25
    fk = SIZE(absc)
    kk = SIZE(absc) / 2
    CALL bessel_zero(absc)
    DO is = 1, kk
       ! First approximation to xz.
       xz = COS(absc(is) / SQRT((fk + 0.5)**2 + c))

       iter = 0
       avsp = 0.0
       DO WHILE (avsp < eps)
          pkm2 = 1.0
          pkm1 = xz
          iter = iter + 1
          IF (iter > 10) CALL fatal(ERR_INTERNAL, &
               'gaussian_grid: no convergence in 10 iterations')

          ! Computation of Legendre polynomial.
          DO n = 2, SIZE(absc)
             fn = n
             pk = ((2.0 * fn - 1.0) * xz * pkm1 - (fn - 1.0) * pkm2) / fn
             pkm2 = pkm1
             pkm1 = pk
          END DO
          pkm1 = pkm2
          sp = pk / ((fk * (pkm1 - xz * pk)) / (1.0 - xz**2))
          xz = xz - sp
          avsp = ABS(sp)
       END DO
       absc(is) = xz
       IF (PRESENT(wght)) wght(is) = (2.0 * (1.0 - xz**2)) / (fk * pkm1)**2
    END DO

    ! Complete abscissas using symmetry.

    IF (SIZE(absc) /= kk * 2) THEN
       absc(kk + 1) = 0.0
       IF (PRESENT(wght)) THEN
          pk = 2.0 / fk**2
          DO n = 2, SIZE(absc), 2
             fn = n
             pk = pk * fn**2 / (fn - 1.0)**2
          END DO
          wght(kk + 1) = pk
       END IF
    END IF
    absc(SIZE(absc) : SIZE(absc) + 1 - kk : -1) = -absc(1 : kk)
    IF (PRESENT(wght)) &
         wght(SIZE(wght) : SIZE(wght) + 1 - kk : -1) = wght(1 : kk)
    absc = -ASIN(absc) / pi * 180.0

    RETURN

  END SUBROUTINE gaussian_grid

  !--------------------------------------------------------------------
  !
  !  Return n zeros, or if n > 50 approximate zeros, of the Bessel
  !  functions j0, in the array bes.  The first 50 zeros will be given
  !  exactly, and the remaining zeros will be computed by extrapolation,
  !  and therefore not exact.
  !

  SUBROUTINE bessel_zero(bes)

    IMPLICIT NONE

    REAL, DIMENSION(:) :: bes

    REAL, PARAMETER :: pi = 3.14159265358979
    INTEGER :: j, nn
    REAL, PARAMETER, DIMENSION(50) :: bz = &
         (/ 2.4048255577,   5.5200781103,   8.6537279129,  11.7915344391, &
           14.9309177086,  18.0710639679,  21.2116366299,  24.3524715308, &
           27.4934791320,  30.6346064684,  33.7758202136,  36.9170983537, &
           40.0584257646,  43.1997917132,  46.3411883717,  49.4826098974, &
           52.6240518411,  55.7655107550,  58.9069839261,  62.0484691902, &
           65.1899648002,  68.3314693299,  71.4729816036,  74.6145006437, &
           77.7560256304,  80.8975558711,  84.0390907769,  87.1806298436, &
           90.3221726372,  93.4637187819,  96.6052679510,  99.7468198587, &
          102.8883742542, 106.0299309165, 109.1714896498, 112.3130502805, &
          115.4546126537, 118.5961766309, 121.7377420880, 124.8793089132, &
          128.0208770059, 131.1624462752, 134.3040166383, 137.4455880203, &
          140.5871603528, 143.7287335737, 146.8703076258, 150.0118824570, &
          153.1534580192, 156.2950342685 /)

    nn = SIZE(bes)
    IF (SIZE(bes) > 50) THEN
       bes(50) = bz(50)
       DO J = 51, SIZE(bes)
          bes(j) = bes(j-1) + pi
       END DO
       nn = 49
    END IF
    DO j = 1, nn
       bes(j) = bz(j)
    END DO

    RETURN

  END SUBROUTINE bessel_zero


  SUBROUTINE regrid(in, inx, iny, out, outx, outy)

    IMPLICIT NONE

    REAL, DIMENSION(:, :),        INTENT(IN)  :: in
    REAL, DIMENSION(SIZE(in, 1)), INTENT(IN)  :: inx
    REAL, DIMENSION(SIZE(in, 2)), INTENT(IN)  :: iny
    REAL, DIMENSION(:, :),        INTENT(OUT) :: out
    REAL, DIMENSION(SIZE(out, 1)), INTENT(IN) :: outx
    REAL, DIMENSION(SIZE(out, 2)), INTENT(IN) :: outy


    INTEGER, DIMENSION(SIZE(out, 1)) :: xbrack
    INTEGER, DIMENSION(SIZE(out, 2)) :: ybrack

    INTEGER :: inidx, outidx
    INTEGER :: row, col
    INTEGER :: ixm, ixp, iym, iyp
    REAL :: xfrac, yfrac


    ! Find bracketing index information: brack_x(col) gives the index
    ! into the inx array of the greatest value smaller than corresponding
    ! value in the outx array.  brack_x(col) and brack_x(col) + 1 thus
    ! give the indices into the inx array of the points bracketing the
    ! output point, i.e. the points that should be used for interpolation.
    ! Similarly for the y values. 

    inidx = 1
    DO outidx = 1, SIZE(out, 1)
       DO WHILE (inidx <= SIZE(in, 1) - 2 .AND. inx(inidx) < outx(outidx))
          inidx = inidx + 1
       END DO
       xbrack(outidx) = inidx
    END DO

    inidx = 1
    DO outidx = 1, SIZE(out, 2)
       DO WHILE (inidx <= SIZE(in, 2) - 2 .AND. iny(inidx) < outy(outidx))
          inidx = inidx + 1
       END DO
       ybrack(outidx) = inidx
    END DO


    ! Do bilinear interpolation, by taking the input x and y values
    ! bracketing the output point (i.e. the corner points of the
    ! smallest rectangle defined by the input axes that contains the
    ! output point), and using the fractional x and y distances of the
    ! output point across the bracketing intervals as interpolation
    ! fractions.

    DO row = 1, SIZE(out, 2)
       iym = ybrack(row)
       iyp = ybrack(row) + 1
       yfrac = (outy(row) - iny(iym)) / (iny(iyp) - iny(iym))
       DO col = 1, SIZE(out, 1)
          ixm = xbrack(col)
          ixp = xbrack(col) + 1
          xfrac = (outx(col) - inx(ixm)) / (inx(ixp) - inx(ixm));
          out(col, row) = &
               in(ixm, iym) * (1 - xfrac) * (1 - yfrac) + &
               in(ixp, iym) * xfrac       * (1 - yfrac) + &
               in(ixm, iyp) * (1 - xfrac) * yfrac       + &
               in(ixp, iyp) * xfrac       * yfrac
       END DO
    END DO

  END SUBROUTINE regrid

END MODULE math_utils

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
