!----------------------------------------------------------------------
!
!  File:   vars_2d.f90
!  Author: Ian Ross, University of Bristol
!  Date:   19 Feb 2005
!
!----------------------------------------------------------------------
!
!  Routines for fetching and calculating 2-D output from FOAM/LPJ.
!
!----------------------------------------------------------------------

MODULE vars_2d

  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils

  USE foam_defs                 ! FOAM specific definitions.
  USE file_utils
  USE ocean_utils
  USE math_utils, ONLY: lat_lon_grid
  USE ice_vars

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: get_one_slice_2d

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 2-D data.
  !

  SUBROUTINE get_one_slice_2d(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    INTEGER :: nc_id

    SELECT CASE (var%name)

       ! Time-invariant data comes from somewhere else for FOAM.

    CASE ('orog', 'sftlf', 'sftgif')
       CALL get_fixed_data(var, axes, data)

       ! These all need to be summed over PFT.

    CASE ('vcarcont', 'npp', 'lcarcont', 'gpp', 'trans', 'potevap')
       CALL get_pft_sum(var, sday, eday, axes, data)

       ! The sum of convective and local precipitation.

    CASE ('pr')
       CALL get_total_precip(var, sday, eday, axes, data)

       ! Net heat flux available as sum of other items.

    CASE ('hfns')
       CALL get_total_surf_hflx(var, sday, eday, axes, data)

       ! Net radiation flux available as sum of other items.

    CASE ('rtmt')
       CALL get_toa_net_rad(var, sday, eday, axes, data)

       ! Ocean surface fields: can be derived from 3-D fields.

    CASE ('sos', 'tos', 'uos', 'vos')
       CALL get_ocean_surf(var, sday, eday, axes, data)

       ! Ocean transport fields require calculation.

    CASE ('hfogo')
       CALL get_ocean_heat_xport(sday, eday, data)

    CASE ('sltov')
       CALL get_ocean_salt_xport(sday, eday, data)

       ! Ocean mixed layer depth.

    CASE ('zmlo')
       CALL get_ocean_mixed_layer(var, sday, eday, axes, data)

       ! Ice base heat flux.

    CASE ('hfsib')
       CALL get_sea_ice_basal_heat_flux(sday, eday, data)

       ! The surface temperature for sea ice has to be handled a little
       ! differently.

    CASE ('ts')
       IF (var%domain == DOMAIN_ICE) THEN
          CALL get_sea_ice_surf_temp(sday, eday, data)
       ELSE
          nc_id = open_history_file(foam_var_name(var%name), eday)
          CALL read_2d_netcdf_var(nc_id, foam_var_name(var%name), data)
          CALL close_file(nc_id)
       END IF

       ! The relative humidity calculation in FOAM is a tiny bit
       ! flakey....

    CASE ('hurs')
       nc_id = open_history_file(foam_var_name(var%name), eday)
       CALL read_2d_netcdf_var(nc_id, foam_var_name(var%name), data)
       CALL close_file(nc_id)
       WHERE (data /= FOAM_MISSING_VAL .AND. data > 100.0) data = 100.0

       ! Others are all simple time-dependent 2-D fields.

    CASE DEFAULT
       nc_id = open_history_file(foam_var_name(var%name), eday)
       CALL read_2d_netcdf_var(nc_id, foam_var_name(var%name), data)
       CALL close_file(nc_id)

    END SELECT
  END SUBROUTINE get_one_slice_2d


  !--------------------------------------------------------------------
  !
  !  Get FOAM time-invariant data.
  !

  SUBROUTINE get_fixed_data(var, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data

    data = 0
    CALL fatal(ERR_UNSUPPORTED, 'get_fixed_data')

  END SUBROUTINE get_fixed_data


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for PFT-summed 2-D variables.
  !

  SUBROUTINE get_pft_sum(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    INTEGER :: nc_id
    CHARACTER(LEN=VAR_NAME_LEN) :: threed_var
    REAL, DIMENSION(SIZE(data, 1), SIZE(data, 2), FOAM_NPFT) :: tmp_data


    ! Get the name of the three-dimensional per PFT variable corresponding
    ! to the two-dimensional total we want.

    threed_var = TRIM(var%name) // 'pft'


    ! Open the file with the three-dimensional data and get the data.

    nc_id = open_history_file(foam_var_name(threed_var), eday)
    CALL read_3d_netcdf_var(nc_id, foam_var_name(threed_var), tmp_data)
    CALL close_file(nc_id)


    ! Accumulate the data across PFTs.

    data = FOAM_MISSING_VAL
    WHERE (tmp_data(:, :, 1) /= FOAM_MISSING_VAL) &
         data = SUM(tmp_data, DIM=3)

  END SUBROUTINE get_pft_sum


  !--------------------------------------------------------------------
  !
  !  Get total precipitation field.
  !

  SUBROUTINE get_total_precip(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Temporary data.
    REAL, DIMENSION(SIZE(data,1), SIZE(data,2)) :: tmp_data

    INTEGER :: nc_id


    ! Read in the convective and local precipitation values.  The total
    ! precipitation is just the sum of these two.

    nc_id = open_history_file('prc', eday)
    CALL read_2d_netcdf_var(nc_id, 'prc', data)
    CALL read_2d_netcdf_var(nc_id, 'prl', tmp_data)
    data = data + tmp_data
    CALL close_file(nc_id)

  END SUBROUTINE get_total_precip


  !--------------------------------------------------------------------
  !
  !  Get total surface heat flux.
  !

  SUBROUTINE get_total_surf_hflx(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Temporary data.
    REAL, DIMENSION(SIZE(data,1), SIZE(data,2)) :: tmp_data

    INTEGER :: nc_id


    ! The net surface heat flux is given by the balance of short-wave and
    ! long-wave radiation at the surface, minus the sensible and latent
    ! heat fluxes.

    nc_id = open_history_file('rss', eday)
    CALL read_2d_netcdf_var(nc_id, 'rss', data)
    CALL read_2d_netcdf_var(nc_id, 'rls', tmp_data)
    data = data + tmp_data
    CALL read_2d_netcdf_var(nc_id, 'hfls', tmp_data)
    data = data - tmp_data
    CALL read_2d_netcdf_var(nc_id, 'hfss', tmp_data)
    data = data - tmp_data
    CALL close_file(nc_id)

  END SUBROUTINE get_total_surf_hflx


  !--------------------------------------------------------------------
  !
  !  Get TOA net radiation field.
  !

  SUBROUTINE get_toa_net_rad(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Temporary data.
    REAL, DIMENSION(SIZE(data,1), SIZE(data,2)) :: tmp_data

    INTEGER :: nc_id


    ! The net radiation at the model top is just the net short-wave flux
    ! at TOA minus the up-welling long-wave at TOA.

    nc_id = open_history_file('rst', eday)
    CALL read_2d_netcdf_var(nc_id, 'rst', data)
    CALL read_2d_netcdf_var(nc_id, 'rlut', tmp_data)
    data = data - tmp_data
    CALL close_file(nc_id)

  END SUBROUTINE get_toa_net_rad


  !--------------------------------------------------------------------
  !
  !  Get ocean surface fields.
  !

  SUBROUTINE get_ocean_surf(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    INTEGER :: nc_id
    CHARACTER(LEN=VAR_NAME_LEN) :: threed_var


    ! Get the name of the three-dimensional variable corresponding to
    ! the two-dimensional surface one we want.

    SELECT CASE (var%name)
    CASE ('tos')
       threed_var = 'thetao'
    CASE ('sos')
       threed_var = 'so'
    CASE ('uos')
       threed_var = 'uo'
    CASE ('vos')
       threed_var = 'vo'
    END SELECT


    ! Open the file with the three-dimensional data and get the surface
    ! slice.

    nc_id = open_history_file(threed_var, eday)
    CALL read_2d_netcdf_slice(nc_id, threed_var, FOAM_ODPTH, data)
    CALL close_file(nc_id)

  END SUBROUTINE get_ocean_surf


  !--------------------------------------------------------------------
  !
  !  Get ocean transport fields.
  !

  SUBROUTINE get_ocean_heat_xport(sday, eday, data)

    IMPLICIT NONE

    ! Day to get.
    INTEGER,                      INTENT(IN)  :: sday, eday

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Longitude/latitude grid.
    REAL, DIMENSION(:), ALLOCATABLE, SAVE    :: lon_vals
    REAL, DIMENSION(:, :), ALLOCATABLE, SAVE :: lon_bnds
    LOGICAL, SAVE                            :: first = .TRUE.

    ! Specific heat capacity of sea water.
    REAL, PARAMETER :: C_p = 4.1E3 ! J kg^-1 K^-1

    ! Temporary data.
    INTEGER :: nc_id
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: thetao
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: so
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: vo

    ! Ocean layer thicknesses.
    REAL, DIMENSION(FOAM_ODPTH) :: thickness = &
         (/ 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, &
            400.0, 400.0, 385.0, 325.0, 230.0, 152.0, 102.0,  70.0, &
             51.0,  40.0,  32.0,  27.0,  24.0,  22.0,  20.0,  20.0  /)


    ! Temporary variables, loop counters and accumulators.
    INTEGER :: ix, iy, iz
    REAL :: t, s, v, rho
    REAL :: dlon, single_col


    ! Get the ocean data we need for this.

    nc_id = open_history_file('thetao', eday)
    CALL read_3d_netcdf_var(nc_id, 'thetao', thetao)
    CALL read_3d_netcdf_var(nc_id, 'so', so)
    CALL read_3d_netcdf_var(nc_id, 'vo', vo)
    CALL close_file(nc_id)


    ! Get longitude values and bounds.

    IF (first) THEN
       CALL lat_lon_grid(DOMAIN_OCEAN, AXIS_LON, lon_vals, lon_bnds)
       first = .FALSE.
    END IF


    ! The outer loop over latitudes fills in latitude points of the YRT output
    ! one at a time.  The inner loop over longitude points is the zonal
    ! integration to get the total meridional heat transport.

    data = FOAM_MISSING_VAL
    DO iy = 1, FOAM_OLAT
       DO ix = 1, FOAM_OLON
          ! Skip land areas.

          IF (thetao(ix, iy, FOAM_ODPTH) == FOAM_MISSING_VAL) CYCLE


          ! Vertical integral of C_p rho T v at a single lon./lat. point.

          single_col = 0
          DO iz = 1, FOAM_ODPTH
             IF (thetao(ix, iy, iz) == FOAM_MISSING_VAL) CYCLE
             t = thetao(ix, iy, iz)
             s = so(ix, iy, iz)
             v = vo(ix, iy, iz)
             rho = eos(t, s, iz) + RHO_REF_OCEAN
             single_col = single_col + C_p * t * rho * v * thickness(iz)
          END DO


          ! Zonal integration factors.

          dlon = D_TO_R * ABS(lon_bnds(2, ix) - lon_bnds(1, ix))
          single_col = single_col * R_EARTH * 1.0E3 * dlon


          ! For each region where the lon./lat. point is present, accumlate
          ! the data.

          IF (point_is_in(REGION_ATLANTIC, ix, ix)) THEN
             IF (data(iy, REGION_ATLANTIC) == FOAM_MISSING_VAL) THEN
                data(iy, REGION_ATLANTIC) = single_col
             ELSE
                data(iy, REGION_ATLANTIC) = &
                     data(iy, REGION_ATLANTIC) + single_col
             END IF
          END IF
          IF (point_is_in(REGION_INDIAN, ix, ix)) THEN
             IF (data(iy, REGION_INDIAN) == FOAM_MISSING_VAL) THEN
                data(iy, REGION_INDIAN) = single_col
             ELSE
                data(iy, REGION_INDIAN) = &
                     data(iy, REGION_INDIAN) + single_col
             END IF
          END IF
          IF (point_is_in(REGION_PACIFIC, ix, ix)) THEN
             IF (data(iy, REGION_PACIFIC) == FOAM_MISSING_VAL) THEN
                data(iy, REGION_PACIFIC) = single_col
             ELSE
                data(iy, REGION_PACIFIC) = &
                     data(iy, REGION_PACIFIC) + single_col
             END IF
          END IF
          IF (data(iy, REGION_GLOBAL) == FOAM_MISSING_VAL) THEN
             data(iy, REGION_GLOBAL) = single_col
          ELSE
             data(iy, REGION_GLOBAL) = &
                  data(iy, REGION_GLOBAL) + single_col
          END IF
       END DO
    END DO

  END SUBROUTINE get_ocean_heat_xport


  SUBROUTINE get_ocean_salt_xport(sday, eday, data)

    IMPLICIT NONE

    ! Day to get.
    INTEGER,                      INTENT(IN)  :: sday, eday

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Longitude/latitude grid.
    REAL, DIMENSION(:), ALLOCATABLE, SAVE    :: lon_vals
    REAL, DIMENSION(:, :), ALLOCATABLE, SAVE :: lon_bnds
    LOGICAL, SAVE                            :: first = .TRUE.

    ! Temporary data.
    INTEGER :: nc_id
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: so
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: vo

    ! Ocean layer thicknesses.
    REAL, DIMENSION(FOAM_ODPTH) :: thickness = &
         (/ 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, &
            400.0, 400.0, 385.0, 325.0, 230.0, 152.0, 102.0,  70.0, &
             51.0,  40.0,  32.0,  27.0,  24.0,  22.0,  20.0,  20.0  /)


    ! Temporary variables, loop counters and accumulators.
    INTEGER :: ix, iy, iz
    REAL :: s, v
    REAL :: dlon, single_col


    ! Get the ocean data we need for this.

    nc_id = open_history_file('so', eday)
    CALL read_3d_netcdf_var(nc_id, 'so', so)
    CALL read_3d_netcdf_var(nc_id, 'vo', vo)
    CALL close_file(nc_id)


    ! Get longitude values and bounds.

    IF (first) THEN
       CALL lat_lon_grid(DOMAIN_OCEAN, AXIS_LON, lon_vals, lon_bnds)
       first = .FALSE.
    END IF


    ! The outer loop over latitudes fills in latitude points of the YRT output
    ! one at a time.  The inner loop over longitude points is the zonal
    ! integration to get the total meridional salt transport.

    data = FOAM_MISSING_VAL
    DO iy = 1, FOAM_OLAT
       DO ix = 1, FOAM_OLON
          ! Skip land areas.

          IF (so(ix, iy, FOAM_ODPTH) == FOAM_MISSING_VAL) CYCLE


          ! Vertical integral of S v at a single lon./lat. point.

          single_col = 0
          DO iz = 1, FOAM_ODPTH
             IF (so(ix, iy, iz) == FOAM_MISSING_VAL) CYCLE
             s = so(ix, iy, iz)
             v = vo(ix, iy, iz)
             single_col = single_col + s * v * thickness(iz)
          END DO


          ! Zonal integration factors.

          dlon = D_TO_R * ABS(lon_bnds(2, ix) - lon_bnds(1, ix))
          single_col = single_col * R_EARTH * 1.0E3 * dlon


          ! For each region where the lon./lat. point is present, accumlate
          ! the data.

          IF (point_is_in(REGION_ATLANTIC, ix, ix)) THEN
             IF (data(iy, REGION_ATLANTIC) == FOAM_MISSING_VAL) THEN
                data(iy, REGION_ATLANTIC) = single_col
             ELSE
                data(iy, REGION_ATLANTIC) = &
                     data(iy, REGION_ATLANTIC) + single_col
             END IF
          END IF
          IF (point_is_in(REGION_INDIAN, ix, ix)) THEN
             IF (data(iy, REGION_INDIAN) == FOAM_MISSING_VAL) THEN
                data(iy, REGION_INDIAN) = single_col
             ELSE
                data(iy, REGION_INDIAN) = &
                     data(iy, REGION_INDIAN) + single_col
             END IF
          END IF
          IF (point_is_in(REGION_PACIFIC, ix, ix)) THEN
             IF (data(iy, REGION_PACIFIC) == FOAM_MISSING_VAL) THEN
                data(iy, REGION_PACIFIC) = single_col
             ELSE
                data(iy, REGION_PACIFIC) = &
                     data(iy, REGION_PACIFIC) + single_col
             END IF
          END IF
          IF (data(iy, REGION_GLOBAL) == FOAM_MISSING_VAL) THEN
             data(iy, REGION_GLOBAL) = single_col
          ELSE
             data(iy, REGION_GLOBAL) = &
                  data(iy, REGION_GLOBAL) + single_col
          END IF
       END DO
    END DO

  END SUBROUTINE get_ocean_salt_xport


  !--------------------------------------------------------------------
  !
  !  Get ocean mixed layer depth.
  !

  SUBROUTINE get_ocean_mixed_layer(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Temporary data.
    INTEGER :: nc_id
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: tdata
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: sdata


    nc_id = open_history_file('thetao', eday)
    CALL read_3d_netcdf_var(nc_id, 'thetao', tdata)
    CALL read_3d_netcdf_var(nc_id, 'so', sdata)
    CALL close_file(nc_id)

    CALL calc_ocean_mixed_layer(tdata, sdata, data)

  END SUBROUTINE get_ocean_mixed_layer

END MODULE vars_2d

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
