!----------------------------------------------------------------------
!
!  File:   model_main.f90
!  Author: Rupert Gladstone, University of Bristol
!  Date:   12 Mar 2005
!
!  modified from code written by Ian Ross
!----------------------------------------------------------------------
!
!  Model-specific code for MOTIF/PMIP2 post-processor for UKMO
!  model, HADCM3.
!
!  There are five routines that the model-specific code needs to
!  support:
!
!-----------------------------------------------------------------------
!  model_init(cfg_args, cfg_entries, description, cal_info, missing)   !
!                                                                      !
!  TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS), INTENT(IN)  :: cfg_args     !
!  INTEGER,                                INTENT(IN)  :: cfg_entries  !
!  CHARACTER(LEN=ARG_LEN),                 INTENT(OUT) :: description  !
!  TYPE(calendar_info),                    INTENT(OUT) :: cal_info     !
!  REAL,                                   INTENT(OUT) :: missing      !
!-----------------------------------------------------------------------
!
!    Initialise the model-specific code.  The information read from the
!    configuration file is passed in in the cfg_args and cfg_entries
!    arguments - this information should be used to set up any internal
!    information needed by the model-specific code to function. 
!    Additionally, the description and cal_info arguments should be
!    filled in with a textual description of the model and information
!    about the calendar used, and missing should be set to the missing
!    value used in the model data.
!
!----------------------------------------------------------------------
!  model_get_axis(axis)                   !
!                                         !
!  TYPE(axis_def), INTENT(INOUT) :: axis  !
!------------------------------------------
!
!   Fill in the definition for a named axis appropriate to the
!   specific model.  The axis_def object passed in has the axis name
!   filled in, but information about the values and range of the axis
!   must be completed by the model_get_axis implementation.  The
!   possible values of the name field in the axis_def object are
!   'longitude', 'latitude', 'depth' (meaning either ocean depth or
!   soil depth), and 'pft'.  The domain (atmosphere, ocean, surface,
!   ice) is determined by the domain member of axis, and should be
!   used to distinguish between, for instance, different latitude/longitude
!   grids used for the atmosphere and ocean.
!
!----------------------------------------------------------------------
!  FUNCTION model_get_units(var_name)                          !
!                                                              !
!  CHARACTER(LEN=UNIT_NAME_LEN)            :: model_get_units  !
!  CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN) :: var_name         !
!---------------------------------------------------------------
!
!   Return the units used by the model (as a udunits compatible unit
!   string) for variable var_name.
!
!----------------------------------------------------------------------
!  FUNCTION model_get_positive(var_name)                     !
!                                                            !
!  CHARACTER(LEN=ARG_LEN)             :: model_get_positive  !
!  CHARACTER(LEN=ARG_LEN), INTENT(IN) :: var_name            !
!-------------------------------------------------------------
!
!   Return the positive direction ('up' or 'down') for a given flux
!   variable.
!
!----------------------------------------------------------------------
!  model_get_value(var, sday, eday, data)   ! <--- Single values
!                                           !
!  TYPE(variable_def), INTENT(IN)  :: var   !
!  REAL,               INTENT(IN)  :: sday  !
!  REAL,               INTENT(IN)  :: eday  !
!  REAL,               INTENT(OUT) :: data  !
!------------------------------------------------------
!  model_get_value(var, sday, eday, axes, data)       ! <--- Hemispheric values
!                                                     !
!  TYPE(variable_def),           INTENT(IN)  :: var   !
!  REAL,                         INTENT(IN)  :: sday  !
!  REAL,                         INTENT(IN)  :: eday  !
!  TYPE(axis_def), DIMENSION(1), INTENT(IN)  :: axes  !
!  REAL, DIMENSION(2),           INTENT(OUT) :: data  !
!------------------------------------------------------
!  model_get_value(var, sday, eday, axes, data)       ! <--- 2-D data
!                                                     !
!  TYPE(variable_def),           INTENT(IN)  :: var   !
!  REAL,                         INTENT(IN)  :: sday  !
!  REAL,                         INTENT(IN)  :: eday  !
!  TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes  !
!  REAL, DIMENSION(:, :),        INTENT(OUT) :: data  !
!------------------------------------------------------
!  model_get_value(var, sday, eday, axes, data)       ! <--- 3-D data
!                                                     !
!  TYPE(variable_def),           INTENT(IN)  :: var   !
!  REAL,                         INTENT(IN)  :: sday  !
!  REAL,                         INTENT(IN)  :: eday  !
!  TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes  !
!  REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data  !
!------------------------------------------------------
!  
!   Set up data array or data value for a given variable for a single
!   timestep.  Four different calling sequences must be supported by
!   the model_get_value interface: one for single value variables
!   (for instance, sifram), one for hemispheric values (for instance,
!   siarea), one for 2-D data, and one for 3-D data.  2- and 3-D data
!   can be defined on any of the possible axes, not just the obvious
!   long./lat./pressure grid - for instance, XYV data is 3-D, YRT
!   data is 2-D, and so on.
!
!----------------------------------------------------------------------

MODULE model_main

  USE proc_utils
  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils
  USE ukmo_defs                 ! FOAM/LPJ specific definitions.
  USE vars_2d
  USE vars_3d
!  USE ice_vars
  USE file_utils
!  USE math_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: model_init
  PUBLIC :: model_get_axis
  PUBLIC :: model_get_units
  PUBLIC :: model_get_positive
  PUBLIC :: model_get_value

  INTERFACE model_get_value
     MODULE PROCEDURE get_val_single ! 1T
     MODULE PROCEDURE get_val_1d     ! 2HT
     MODULE PROCEDURE get_val_2d     ! XY, XYT, YRT
     MODULE PROCEDURE get_val_3d     ! XYPT, XYST, XYVT, XYZT, YZRT
  END INTERFACE

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Initialise the FOAM/LPJ model interface.  Model-specific command
  !  line arguments are available in the command_line array.
  !

  SUBROUTINE model_init(cfg_args, cfg_entries, description, cal_info, missing)

    IMPLICIT NONE

    TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS), INTENT(IN)  :: cfg_args
    INTEGER,                                INTENT(IN)  :: cfg_entries

    CHARACTER(LEN=ARG_LEN),                 INTENT(OUT) :: description
    TYPE(calendar_info),                    INTENT(OUT) :: cal_info
    REAL,                                   INTENT(OUT) :: missing


    ! Set up model description and calendar information.

    description = 'UKMO HADCM3 ' // &
         'atmosphere: 73*96; ' // &
         'ocean: 144*288; ' // &
         'land: MOSES II/TRIFFID'
    cal_info%name = '360_day'
    cal_info%mon_lens = (/ 30, 30, 30, 30, 30, 30, &
                           30, 30, 30, 30, 30, 30  /)
    missing = UKMO_MISSING_VAL


    ! Extract mandatory configuration entries for FOAM/LPJ.

    ukmo_data_dir = get_cfg_value(cfg_args, cfg_entries, 'ukmo_data_dir')
    IF (TRIM(ukmo_data_dir) == '') CALL fatal(ERR_CFG_MISSING, 'ukmo_data_dir')
    CALL normalise_dir(ukmo_data_dir)
    ukmo_job = get_cfg_value(cfg_args, cfg_entries, 'ukmo_job')
    IF (TRIM(ukmo_job) == '') CALL fatal(ERR_CFG_MISSING, 'ukmo_job')


    ! Extract optional information from the configuration file
    ! (foam_fixed_dir is only needed to get things like the orography, so
    ! it's not mandatory).

!    regions_file = get_cfg_value(cfg_args, cfg_entries, 'foam_regions')

  END SUBROUTINE model_init


  !--------------------------------------------------------------------
  !
  !  Fill in a given axis definition object from the axis name and
  !  domain.  Since FOAM uses the same latitude grid for ocean,
  !  surface and ice models, there are only two possible results for
  !  latitude and longitude axes.  Ocean and soil depth axes are
  !  treated correctly, as is the PFT axis.
  !  
  !  NOTE: THE cell_bounds INFORMATION *MUST* BE SUPPLIED FOR LONGITUDE
  !        AND LATITUDE AXES.  CMOR WILL CRASH IF YOU DON'T SUPPLY THIS   
  !        INFORMATION!
  !
       
  SUBROUTINE model_get_axis(axis,var)
    
    IMPLICIT NONE
       
    ! The domain and name elements of the axis definition are set up
    ! on entry, while the values entry is unallocated.  If the domain
    ! and name are appropriate, model_get_axis will allocate and fill
    ! the values entry.

    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN) :: var
                  
    TYPE(axis_def),           INTENT(INOUT) :: axis
                  
    REAL, DIMENSION(1)         :: p_wmom  ! WMO pressure levels
    REAL, DIMENSION(3)         :: p_wmo3  !
    REAL, DIMENSION(5)         :: pft     ! plant functional types
    REAL, DIMENSION(9)         :: tiles
    REAL, DIMENSION(8)         :: nitiles ! non-ice tiles    
    REAL, DIMENSION(20)        :: ocn_dpth1
    REAL, DIMENSION(19)        :: ocn_dpth2
    REAL, DIMENSION(288)       :: ocn_lon1, ocn_lon2
    REAL, DIMENSION(144)       :: ocn_lat1
    REAL, DIMENSION(143)       :: ocn_lat2
    REAL, DIMENSION(96)        :: atm_lon1,atm_lon2
    REAL, DIMENSION(72)        :: atm_lat2
    REAL, DIMENSION(73)        :: atm_lat1

    INTEGER :: i


    pft = (/ 1, 2, 3, 4, 5 /)

    nitiles = (/ 1, 2, 3, 4, 5, 6, 7, 8 /)

    tiles = (/ 1, 2, 3, 4, 5, 6, 7, 8, 9 /)

    ocn_dpth1=(/ 5.00000,   15.0000,   25.0000,  35.1000,  47.8500,  67.0000, &
                 95.7500,   138.900,   203.700,  301.000,  447.050,  666.300, &
                 995.550,   1500.85,   2116.15,  2731.40,  3346.65,  3961.90, &
                 4577.15,   5192.45    /)

    ! half levels, for vert vel only
    ocn_dpth2=(/ 10. , &   
         20.     ,   30.   ,   40.2  ,   55.5  , &
         78.5    ,  113.   ,  164.8  ,  242.60 , &
        359.40   ,  534.70 ,  797.9  , 1193.20 , &
       1808.5    , 2423.80 , 3039.   , 3654.30 , &
       4269.5    , 4884.80  /)

    ocn_lon1 = (/ ( (i - 1) * 1.25, i = 1, 288 ) /)
    ocn_lon2 = (/ ( 0.625 + (i - 1) * 1.25, i = 1, 288 ) /)

    ocn_lat1 = (/ ( -89.375 + (i - 1) * 1.25, i = 1, 144 ) /)
    ocn_lat2 = (/ ( -88.75 + (i - 1) * 1.25, i = 1, 143 ) /)

    atm_lat1 = (/ ( 90 - (i - 1) * 2.5, i = 1, 73 ) /)
    atm_lat2 = (/ ( 88.75 - (i - 1) * 2.5, i = 1, 72 ) /)

    atm_lon1 = (/ ( (i - 1) * 3.75, i = 1, 96 ) /)
    atm_lon2 = (/ ( 1.875 + (i - 1) * 3.75, i = 1, 96 ) /)

    p_wmo3= (/ 850., 500., 200. /)

    p_wmom= (/ 500. /)


    SELECT CASE (axis%name)



    CASE (AXIS_LON)
    
        IF (axis%domain == DOMAIN_ATMOS .OR. axis%domain == DOMAIN_SURFACE) THEN
            SELECT CASE (var)
            CASE ('tas','ta','tasmin','tasmax','zg','pr','clivi','clwvi','prw', &
                  'rstcs','rsutcs','rsscs','rsuscs','rst','rsut','rtmt','albs', &
                  'mrros','lcarcont','lcarcontpft','vcarcont','vcarcontpft', &
                  'pftfrac','clt','hfns','cl')
               ALLOCATE(axis%values(size(atm_lon1)))
               axis%values=atm_lon1
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)           
      
            CASE ('ua','va','wa','hus','intuaw','intvaw','tauv','tauu')
               ALLOCATE(axis%values(size(atm_lon2)))
               axis%values=atm_lon2
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)
            END SELECT   

        ELSE IF (axis%domain == DOMAIN_OCEAN) THEN
            SELECT CASE (var)
            CASE ('sisnvol','siprod','sit','wfo','tos','ts',&
                  'hcont300','thetao','ziso20','hfo','hfsib','stfbarot',&
                  'sic','sildarea','wo','sltov','sos','so','zmlo')
               ALLOCATE(axis%values(size(ocn_lon1)))
               axis%values=ocn_lon1
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)
            
            CASE ('usi','vsi','tauv','tauu','uo','vo','vos','uos')
               ALLOCATE(axis%values(size(ocn_lon2)))
               axis%values=ocn_lon2
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds) 
            END SELECT
        END IF


    CASE (AXIS_LAT)   

        IF (axis%domain == DOMAIN_ATMOS .OR. axis%domain == DOMAIN_SURFACE) THEN
            SELECT CASE (var)
            CASE ('tas','ta','tasmin','tasmax','zg','pr','clivi','clwvi','prw', &
                  'rstcs','rsutcs','rsscs','rsuscs','rst','rsut','rtmt','albs', &
                  'mrros','lcarcont','lcarcontpft','vcarcont','vcarcontpft', & 
                  'pftfrac','clt','hfns','cl')
               ALLOCATE(axis%values(size(atm_lat1)))
               axis%values=atm_lat1
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)          
     
            CASE ('ua','va','hus','wa','intuaw','intvaw','tauv','tauu')
               ALLOCATE(axis%values(size(atm_lat2)))
               axis%values=atm_lat2
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)
            END SELECT

        ELSE IF (axis%domain == DOMAIN_OCEAN) THEN
            SELECT CASE (var)
            CASE ('sisnvol','siprod','sit','wfo','tos','ts','zmlo','thetaozm',&
                  'hcont300','hfogo','thetao','ziso20','hfo','hfsib','stfbarot',&
                  'sic','sildarea','stfmmc','wo','sltov','sos','sozm','so')
               ALLOCATE(axis%values(size(ocn_lat1)))
               axis%values=ocn_lat1
               CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)
               
             CASE ('usi','vsi','tauv','tauu','uo','vo','vos','uos')
                ALLOCATE(axis%values(size(ocn_lat2)))
                axis%values=ocn_lat2
                CALL lat_lon_bounds(axis%name, axis%values, axis%bounds)
             END SELECT
        END IF


    CASE (AXIS_DEPTH)

        SELECT CASE (var)
        CASE ('sos','so','sozm','thetaozm','thetao','uo','vo')
           ALLOCATE(axis%values(size(ocn_dpth1)))
           axis%values=ocn_dpth1
          
        CASE ('wo')
           ALLOCATE(axis%values(size(ocn_dpth2)))
           axis%values=ocn_dpth2
          
        END SELECT


    CASE (AXIS_PFT)

        SELECT CASE (var)            
        CASE ('gpppft','laipft','npppft','vcarcontpft','lcarcontpft')
           ALLOCATE(axis%values(size(pft)))
           axis%values=pft

        CASE ('potevappft','hfns','hfss','transpft')
           ALLOCATE(axis%values(size(nitiles)))
           axis%values=nitiles

        CASE ('pftfrac')
           ALLOCATE(axis%values(size(tiles)))  
           axis%values=tiles 

        END SELECT


    CASE (AXIS_HGT1, AXIS_HGT2)
       ALLOCATE(axis%values(1))
       IF (TRIM(axis%name) == AXIS_HGT1) THEN
          axis%values(1) = 1.5
       ELSE
          axis%values(1) = 10.0
       END IF
    
       
    CASE DEFAULT
       CALL fatal(ERR_UNKNOWN_AXIS, axis%name)
       
    END SELECT
  
  END SUBROUTINE model_get_axis

!not yet implemented for UM
  !--------------------------------------------------------------------
  !
  !  Return unit used by model for a given variable.  All of the units
  !  used in FOAM output are identical to those required in the MOTIF
  !  output files.
  !

  FUNCTION model_get_units(var_name) RESULT(units)

    CHARACTER(LEN=UNIT_NAME_LEN)            :: units
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN) :: var_name
    

    SELECT CASE (var_name)

       CASE ('laipft', 'albs', 'fpar', 'pftfrac')
          units = '1'

       CASE ('sftgif', 'sftlf', 'clt', 'sic', 'snc', 'cl')
          units = '1'

       CASE ('hurs', 'hur')
          units = '0.01'

       CASE ('intuadse', 'intvadse')
          units = '1.0E6 J m-1 s-1'

       CASE ('intuaw', 'intvaw')
          units = '1.0E6 kg m-1 s-1'

       CASE ('so', 'sos', 'sozm')
          units = '1.0E-3'

       CASE ('tasmax', 'tasmin', 'tsoil', 'tas', 'ta', 'ts')
          units = 'K'

       CASE ('thetaozm', 'thetao', 'tos')
          units = 'degC'

       CASE ('hus', 'huss')
          units = 'kg kg-1'

       CASE ('prw', 'mrso', 'clivi', 'clwvi')
          units = 'kg m-2'

       CASE ('bioburnt')
          units = 'gram m-2 month-1'

       CASE ('lcarcont', 'scarcont', 'vcarcont', 'lcarcontpft', 'vcarcontpft')
          units = 'kg m-2'

       CASE ('pr', 'prc', 'snm', 'wfo', 'mrro', 'sbls', 'esoil',   &
             'mrros', 'potevap', 'trans', 'evspsbl', 'potevappft', &
             'transpft', 'evspsblveg', 'hresp')
          units = 'kg m-2 s-1'

       CASE ('gpp', 'npp', 'gpppft', 'npppft')
          units = 'gram m-2 s-1'

       CASE ('sltov')
          units = 'kg s-1'

       CASE ('sildarea', 'siarea', 'siext')
          units = 'km2'

       CASE ('sisnvol', 'sivol')
          units = 'km3'

       CASE ('ziso20', 'cheight', 'orog', 'zmlo', 'sit', 'snd', 'zos', 'zg')
          units = 'm'

       CASE ('dis', 'stfmmc', 'stfbarot')
          units = 'm3 s-1'

       CASE ('hcont300')
          units = 'm K'

       CASE ('siprod')
          units = 'm month-1'

       CASE ('ua', 'uo', 'va', 'vo', 'wo', 'uas', 'uos', 'usi', 'vas', &
             'vos', 'vsi', 'uvas')
          units = 'm s-1'

       CASE ('mpuoto', 'mpvoto', 'mpuoeito', 'mpvoeito')
          units = 'm s-1 K'

       CASE ('mpuoso', 'mpvoso', 'mpuoeiso', 'mpvoeiso')
          units = 'm s-1 1.0E-3'

       CASE ('tauu', 'tauv', 'psl', 'ps')
          units = 'Pa'

       CASE ('wap')
          units = 'Pa s-1'

       CASE ('sifram')
          units = '1.0E6 m3 s-1'

       CASE ('hfogo')
          units = 'W'

       CASE ('hfo', 'rls', 'rss', 'rst', 'hfls', 'hfns', 'hfss', 'rlds', &
             'rlus', 'rlut', 'rsds', 'rsdt', 'rsus', 'rsut', 'rtmt',     &
             'hfsib', 'rlscs', 'rsscs', 'rstcs', 'rldscs', 'rlutcs',     &
             'rsdscs', 'rsuscs', 'rsutcs')
          units = 'W m-2'

       END SELECT

  END FUNCTION model_get_units


!not yet implemented for UM
  !--------------------------------------------------------------------
  !
  !  Return positive direction for a flux variable.
  !

  FUNCTION model_get_positive(var_name) RESULT(dir)

    CHARACTER(LEN=ARG_LEN)             :: dir
    CHARACTER(LEN=ARG_LEN), INTENT(IN) :: var_name
    

    ! For FOAM, they're all downward apart from the radiation fluxes
    ! explicitly labelled as upwards, and the latent and sensible heat
    ! fluxes.

    SELECT CASE (TRIM(var_name))

!    CASE ('rlus', 'rlut', 'rsus', 'rsut', 'rlutcs', 'rsuscs', 'rsutcs', &
!          'hfls', 'hfss')


    CASE ('rlus','rsus')
        dir = 'up'

    CASE ('rlscs','rsscs','rstcs','rsyt','rtmt','hfns')
        dir = 'down'



!    CASE DEFAULT
!       dir = 'down'

    END SELECT

  END FUNCTION model_get_positive


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for singleton data.
  !

  SUBROUTINE get_val_single(var, sday, eday, data)

    IMPLICIT NONE

    TYPE(variable_def), INTENT(IN)  :: var
    INTEGER,            INTENT(IN)  :: sday, eday

    REAL,               INTENT(OUT) :: data


    ! UM doesn't handle any of these.
    print*,'failed in get_val_single'

    data = 0
    CALL fatal(ERR_MODEL_INVALID_VAR, var%name)

  END SUBROUTINE get_val_single


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 1-D data.
  !

  SUBROUTINE get_val_1d(var, sday, eday, axes, data)

    IMPLICIT NONE

    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(1), INTENT(IN)  :: axes

    REAL, DIMENSION(:),           INTENT(OUT) :: data

    ! UM doesn't handle any of these.

    data = 0
    print*, 'failed in get_val_1d'
    CALL fatal(ERR_MODEL_INVALID_VAR, var%name)

  END SUBROUTINE get_val_1d


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 2-D data.
  !

  SUBROUTINE get_val_2d(var, sday, eday, axes, data)

    IMPLICIT NONE

    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    REAL, DIMENSION(:,:),         INTENT(OUT) :: data

    IF (eday - sday == 0) THEN
       CALL get_daily_2d_slice(var, sday, axes, data)
    ELSE
       CALL get_monthly_2d_slice(var, sday, axes, data)
    ENDIF

  END SUBROUTINE get_val_2d


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 3-D data.
  !

  SUBROUTINE get_val_3d(var, sday, eday, axes, data)

    IMPLICIT NONE

    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    REAL, DIMENSION(:,:,:),       INTENT(OUT) :: data

    data = 0

    IF (eday - sday == 0) THEN
       CALL get_daily_3d_slice(var, sday, axes, data)
    ELSE
       CALL get_monthly_3d_slice(var, sday, axes, data)
    ENDIF

!    CALL fatal(ERR_MODEL_INVALID_VAR, var%name)

  END SUBROUTINE get_val_3d


! no data routines in place yet...
!    
!    ! information needed in constructing file name
!    CALL get_um_info(var,um_name,out_str,sub_model)
!
!    SELECT CASE (var%name)
! 
!    CASE ('tas')
!        CALL get_3d_data(um_name,sday,eday,'a',out_str,data)
!  
!    CASES:
!    unit conversions to be included
!    awkward ice variables (through fram straight, hemispheric area etc)
!    variables that need summing/subtracting (mainly radiation)
!    hybrid to pressure coordinate conversions
!    monthly min and max for tasmin and tasmax
!    ocean basin calcs
!    overturning streamfunction
!    vert integrals of moisture and dry static energy transports
!    
!    CASE DEFAULT
!       CALL fatal(ERR_MODEL_INVALID_VAR, var%name)
!    END SELECT


END MODULE model_main

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
