!----------------------------------------------------------------------
!
!  File:   vars_2d.f90
!  Author: Some Jokers
!  Date:   12 Mar 2005
!
!----------------------------------------------------------------------
!
!  Routines for fetching and calculating 3-D output from UKMO.
!
!----------------------------------------------------------------------

MODULE vars_3d

  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils

  USE ukmo_defs                 ! FOAM specific definitions.
  USE file_utils
  USE proc_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: get_daily_3d_slice
  PUBLIC :: get_monthly_3d_slice

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 3-D data.
  !

  SUBROUTINE get_monthly_3d_slice(var, sday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :,:),        INTENT(OUT) :: data

    REAL, DIMENSION(:, :,:), ALLOCATABLE :: data_tmp
    CHARACTER(LEN=30) :: um_name
    CHARACTER(LEN=2)  :: out_str
    CHARACTER         :: sub_model
    CHARACTER(LEN=20) :: fname
    CHARACTER(LEN=4)  :: sub_dir
    INTEGER           :: nc_id,n
    
    data = 0

    SELECT CASE (var%name)


    ! cloud amount is only available on hybrid coords.  Need to convert to pressure.

    CASE ('cl')

print*,'sizes before ', SIZE(data,1),SIZE(data,2),SIZE(data,3)

        ALLOCATE(data_tmp(SIZE(data,1),SIZE(data,2),19))
        um_name='cldamount_mm_hyb'
        out_str='pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_3d_netcdf_var(nc_id, um_name, data_tmp)
        CALL close_file(nc_id)
        CALL interp_to_press(data_tmp, data,axes,sday)
print*,'sizes after ', SIZE(data,1),SIZE(data,2),SIZE(data,3)

    CASE ('ta') 
        um_name='temp_mm_p'   
        out_str='pc'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_3d_netcdf_var(nc_id, um_name, data)
        CALL close_file(nc_id)


    ! Triffid data is obtained at 10 day intervals.
    ! Need to average over month

    CASE ('lcarcontpft','vcarcontpft')
        ALLOCATE(data_tmp(SIZE(axes(1)%values), &
                 SIZE(axes(2)%values), SIZE(axes(3)%values)))
        CALL get_um_info(var, um_name, out_str, sub_model)
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)

        ! read and sum the 10-day values
        DO n=1,3 
            CALL read_3d_netcdf_subvar(nc_id, um_name, data_tmp, n)
            WHERE (data_tmp /= UKMO_MISSING_VAL)
                data = data + data_tmp
            ELSEWHERE (data_tmp == UKMO_MISSING_VAL)
                data = UKMO_MISSING_VAL
            END WHERE
        END DO
        CALL close_file(nc_id)

        ! calculate monthly mean
        WHERE (data /= UKMO_MISSING_VAL)
            data = data / 3
        END WHERE

    
    CASE DEFAULT
        CALL get_um_info(var, um_name, out_str, sub_model)
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_3d_netcdf_var(nc_id, um_name, data)
        CALL close_file(nc_id)

    END SELECT

  END SUBROUTINE get_monthly_3d_slice


  !--------------------------------------------------------------------
  !
  !  Get daily value for 3-D data.
  !

  SUBROUTINE get_daily_3d_slice(var, sday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :,:),      INTENT(OUT) :: data


    CHARACTER(LEN=30) :: um_name
    CHARACTER(LEN=2)  :: out_str
    CHARACTER         :: sub_model
    CHARACTER(LEN=20) :: fname
    CHARACTER(LEN=4)  :: sub_dir
    INTEGER           :: nc_id

    SELECT CASE (var%name)
    
    CASE ('ta')
        um_name='temp_dm_p'
        out_str='pa' 
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_3d_netcdf_var(nc_id, um_name, data)
        CALL close_file(nc_id)
        
    CASE DEFAULT
        CALL get_um_info(var, um_name, out_str, sub_model)
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_3d_netcdf_subvar(nc_id, um_name, data, MOD(sday - 1, 30) + 1)
        CALL close_file(nc_id)
        
    END SELECT

  END SUBROUTINE get_daily_3d_slice

END MODULE vars_3d

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
